/*
Logiciel de commande de 10 aiguilles par servo
Par 10 inverseurs 
Associés à 10 paires de Leds (2 par aiguille) pour afficher la position courante

Auteur : Dominique BULTEZ
 *
 * Ce logiciel est distribue sous la licence GNU Public v2 (GPLv2)
 *
 * Please read the LICENCE file
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
*/

// VERSION 1.1 du 1 Decembre 2016

#include <Arduino.h>
#include <Servo.h>
#include <Bounce2.h>
#include <EEPROM.h>
#include "Dix_aiguilles.h"


#define MaxAig 10                   // nombre d'ensembles Aiguille+Inter+2leds


const int tablePins[MaxAig][4] = { // pour chaque aiguille il faut renseigner les No de pins :
  {14, 15, 16 ,17},                // pin Servo, pin Inter, pin Led Directe, pin led Déviée
  {18, 19, 20, 21},                // c'est geroupé par blocs de quatre
  {22, 23, 23, 25},                // correspondant au groupes de 4 connecteurs 
  {26, 27, 28, 29},                // sur la carte fakir
  {30, 31, 32, 33},
  {34, 35, 36, 37},
  {38, 39, 40, 41},
  {42, 43, 44, 45},
  {46, 47, 58, 49},
  {50, 51, 52, 53} 
};

int tablePos[MaxAig][2] = { // butées Min et Max en globales
  {1500, 1600},
  {1500, 1600},
  {1500, 1600},
  {1500, 1600},
  {1500, 1600},
  {1500, 1600},
  {1500, 1600},
  {1500, 1600},
  {1500, 1600},
  {1500, 1600}  
};

CommandeAiguille*  aiguille0=new CommandeAiguille();
CommandeAiguille*  aiguille1=new CommandeAiguille();
CommandeAiguille*  aiguille2=new CommandeAiguille();
CommandeAiguille*  aiguille3=new CommandeAiguille();
CommandeAiguille*  aiguille4=new CommandeAiguille();
CommandeAiguille*  aiguille5=new CommandeAiguille();
CommandeAiguille*  aiguille6=new CommandeAiguille();
CommandeAiguille*  aiguille7=new CommandeAiguille();
CommandeAiguille*  aiguille8=new CommandeAiguille();
CommandeAiguille*  aiguille9=new CommandeAiguille();

CommandeAiguille* tableAiguille[MaxAig] {
  aiguille0, aiguille1, aiguille2, aiguille3, aiguille4, 
  aiguille5, aiguille6, aiguille7, aiguille8, aiguille9
};

/////////// variable globales ///////////
  byte V0, V1;
  int val;
  int numServo, encoursServo;
  bool minmax; // min = false, max = true


/////////////////////////////////////////////////////////
void setup() 
{
  int numServo;

  Serial.begin(115200);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  V0 = EEPROM.read(0);
  V1 = EEPROM.read(1);
  if ((V0!=0xFF)&&(V1!=0xFF)) {
    val = V0;
    val = val << 8;
    val = val | V1;
    Serial.print("Version ");Serial.println(val);
    EEPROM.get( 2, tablePos );  //lecture de la table des min et max.
    for( int j = 0 ; j < MaxAig ; j++ ){
      Serial.print( j ); 
      Serial.print( ':' ); 
      Serial.print( tablePos[ j ][ 0 ]);
      Serial.print( ',' ); 
      Serial.print( tablePos[ j ][ 1 ]);
      Serial.println(); 
    }
  } else {
    Serial.println("EEPROM vierge");
    val = 0;
  }
    for (numServo = 0; numServo < MaxAig; numServo++) {
    tableAiguille[numServo]->SetpinServo(tablePins[numServo][0]);
    tableAiguille[numServo]->SetpinInter(tablePins[numServo][1]);
    tableAiguille[numServo]->SetpinLedDirecte(tablePins[numServo][2]);
    tableAiguille[numServo]->SetpinLedDeviee(tablePins[numServo][3]);
    tableAiguille[numServo]->SetPosMinMax(tablePos[numServo][0], tablePos[numServo][1]);
    tableAiguille[numServo]->InitCommandeAiguille();
  } 

}

//////////////////////////////////////////////////////
void loop() 
{

  for (numServo = 0; numServo < MaxAig; numServo++) {
    tableAiguille[numServo]->ServoUpdate();
    tableAiguille[numServo]->InterUpdate();
  }
  
  while (Serial.available() > 0) 
  {
    int commande = Serial.parseInt();  // lecture entier
    if (Serial.read() == '\n') 
    {
      switch (commande) {
        case 0:
        val = val + 1;
        V1 = val;
        V0 = val >> 8;
        EEPROM.update(0, V0);
        EEPROM.update(1, V1);
        Serial.println("Enregistrement EEPROM");
        EEPROM.put( 2, tablePos );  //Write array data.
        break;
        
        case 1:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 2:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 3:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 4:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 5:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 6:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 7:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 8:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 9:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        case 10:
        encoursServo = commande - 1;
        Serial.print("Servo "); Serial.print(encoursServo+1); Serial.print(" : ");
        Serial.print(tablePos[encoursServo][0]);Serial.print("->");
        minmax = false;
        break;
        
        default:
        if ((commande > 1400) && (commande < 1700)) {
          Serial.print(commande);
          if (!minmax) {
            tablePos[encoursServo][0] = commande;
            minmax = true;
            Serial.print(", ");
            Serial.print(tablePos[encoursServo][1]);Serial.print("->");
          } else {
            tablePos[encoursServo][1] = commande;
            Serial.println();
            Serial.print("Aiguille "); Serial.print(encoursServo+1); Serial.print(" : ");
            Serial.print(tablePos[encoursServo][0]);Serial.print(", ");
            Serial.println(tablePos[encoursServo][1]);
          }
        }
        break;
      }
      
    }
  }

}

//
